function addObservedAttribute(metadata, attributeName) {
    if (!metadata.observedAttributes) {
        metadata.observedAttributes = [];
    }
    metadata.observedAttributes.push(attributeName);
}
function addReactiveAttribute(metadata, attributeName) {
    addObservedAttribute(metadata, attributeName);
    if (!metadata.reactiveAttributes) {
        metadata.reactiveAttributes = new Set();
    }
    metadata.reactiveAttributes.add(attributeName);
}
function addAttributeChangeCallback(metadata, attributeName, callback) {
    if (!metadata.attributeChangeCallbacks) {
        metadata.attributeChangeCallbacks = new Map();
    }
    metadata.attributeChangeCallbacks.set(attributeName, callback);
}
// TODO: an unfortunate amount of 'any' here. Needs fixing but I ran out of time.
/**
 * Define this accessor as a reactive property.
 * This means the render function will run whenever this property (or the associated attribute) changes.
 *
 * @param reflector A reflector to handle getting, setting, and converting to/from attributes.
 * @param options
 */
function reactive(reflector, { attribute, onChange } = {}) {
    return (target, context) => {
        const attributeName = attribute !== null && attribute !== void 0 ? attribute : context.name.toLowerCase();
        if (attributeName) {
            addReactiveAttribute(context.metadata, attributeName);
            if (onChange) {
                addAttributeChangeCallback(context.metadata, attributeName, onChange);
            }
        }
        const setter = (instance, value) => {
            if (!attributeName) {
                // Avoid queuing a render if the value hasn't changed.
                if (context.access.get(instance) === value)
                    return;
                target.set.call(instance, value);
                if (onChange)
                    onChange(instance);
                instance.queueRender();
                return;
            }
            const attributeValue = reflector.toAttribute
                ? reflector.toAttribute(value)
                : value;
            if (attributeValue === null) {
                if (!instance.hasAttribute(attributeName))
                    return;
                instance.removeAttribute(attributeName);
                if (onChange)
                    onChange(instance);
                instance.queueRender();
                return;
            }
            const stringAttributeValue = String(attributeValue);
            if (instance.getAttribute(attributeName) === stringAttributeValue)
                return;
            instance.setAttribute(attributeName, String(attributeValue));
            if (onChange)
                onChange(instance);
            instance.queueRender();
            return;
        };
        return {
            get() {
                let value = undefined;
                if (attributeName) {
                    const attributeValue = this.getAttribute(attributeName);
                    if (attributeValue !== null && reflector.parseAttribute) {
                        value = reflector.parseAttribute(attributeValue);
                    }
                }
                else {
                    value = target.get.call(this);
                }
                if (reflector.get)
                    value = reflector.get(value !== null && value !== void 0 ? value : null, this);
                value !== null && value !== void 0 ? value : (value = reflector.defaultValue);
                return value;
            },
            set(value) {
                reflector.set(value, (newValue) => setter(this, newValue));
            },
            init(value) {
                if (value !== undefined) {
                    console.warn(`Don't initialize reactive properties - use the reflector to handle that. Spotted on ${this.constructor.name} - ${context.name}`);
                }
                return value;
            },
        };
    };
}

/**
 * Register this class as a custom element.
 *
 * @param tagName The HTML tag name to register.
 *
 * @example
 * ```ts
 * @define('ui-button')
 * export default class Button extends PreactCustomElement {
 *   // …
 * }
 * ```
 */
function define(tagName) {
    return (Constructor, context) => {
        // if already defined, don't try to define again and break things
        if ('customElements' in globalThis && !customElements.get(tagName)) {
            // Otherwise, metadata is attached too late:
            Object.defineProperty(Constructor, Symbol.metadata, {
                enumerable: true,
                configurable: true,
                writable: true,
                value: context.metadata,
            });
            customElements.define(tagName, Constructor);
        }
    };
}

export { define, reactive };
