import { createStylesheet, y, __setFunctionName, __esDecorate, __runInitializers, PreactCustomElement, __classPrivateFieldGet, __classPrivateFieldSet } from '../../chunks/index.js';
import { reactive, define } from '../../chunks/define.js';
import { string } from '../../chunks/string.js';
import { stringEnum } from '../../chunks/stringEnum.js';
import { tagName } from './meta.js';
import { layoutStyles } from '../../chunks/layout.js';

/**
 * @returns The HTML element that should be rendered for the given `accessibilityRole`. Defaults to `'div'`.
 */
function getElementFromRole(accessibilityRole) {
    return ACCESSIBILITY_ELEMENT_MAP[accessibilityRole !== null && accessibilityRole !== void 0 ? accessibilityRole : 'generic'];
}
/**
 * @returns The value that should be used in the HTML attribute `role`, for the provided `accessibilityRole`. Returns `undefined` if the role is not included in the map of available roles.
 */
function getResolvedRole(accessibilityRole) {
    if (accessibilityRoleIsRoleKey(accessibilityRole)) {
        return ACCESSIBILITY_ROLE_MAP[accessibilityRole];
    }
    return undefined;
}
const ACCESSIBILITY_ROLE_MAP = {
    'list-item-separator': 'separator',
    separator: 'separator',
    alert: 'alert',
};
const ACCESSIBILITY_ELEMENT_MAP = {
    main: 'main',
    header: 'header',
    footer: 'footer',
    section: 'section',
    navigation: 'nav',
    'ordered-list': 'ol',
    'unordered-list': 'ul',
    'list-item': 'li',
    'list-item-separator': 'li',
    separator: 'hr',
    alert: 'div',
    generic: 'div',
    aside: 'aside',
    status: 'div',
};
/**
 * Narrows the type of `accessibilityRole` to a key in the `ACCESSIBILITY_ROLE_MAP`.
 */
function accessibilityRoleIsRoleKey(accessibilityRole) {
    return !!accessibilityRole && accessibilityRole in ACCESSIBILITY_ROLE_MAP;
}

const componentStyles = createStylesheet(":host{width:100%}.block-stack{display:flex;flex-direction:column;height:100%;width:100%}");

const styles = [componentStyles, layoutStyles];
function ShadowRoot(element) {
    const { accessibilityRole, accessibilityLabel } = element;
    const Tag = getElementFromRole(accessibilityRole);
    const role = getResolvedRole(accessibilityRole);
    const ariaLabel = accessibilityLabel ? accessibilityLabel : undefined;
    return (y(Tag, { role: role, class: "block-stack", "aria-label": ariaLabel },
        y("slot", null)));
}

const shadowRoot = /*#__PURE__*/Object.freeze(/*#__PURE__*/Object.defineProperty({
    __proto__: null,
    ShadowRoot,
    styles
}, Symbol.toStringTag, { value: 'Module' }));

let BlockStack = (() => {
    var _BlockStack_accessibilityRole_accessor_storage, _BlockStack_accessibilityLabel_accessor_storage;
    let _classDecorators = [define(tagName)];
    let _classDescriptor;
    let _classExtraInitializers = [];
    let _classThis;
    let _classSuper = PreactCustomElement;
    let _instanceExtraInitializers = [];
    let _accessibilityRole_decorators;
    let _accessibilityRole_initializers = [];
    let _accessibilityLabel_decorators;
    let _accessibilityLabel_initializers = [];
    _classThis = class extends _classSuper {
        get accessibilityRole() { return __classPrivateFieldGet(this, _BlockStack_accessibilityRole_accessor_storage, "f"); }
        set accessibilityRole(value) { __classPrivateFieldSet(this, _BlockStack_accessibilityRole_accessor_storage, value, "f"); }
        get accessibilityLabel() { return __classPrivateFieldGet(this, _BlockStack_accessibilityLabel_accessor_storage, "f"); }
        set accessibilityLabel(value) { __classPrivateFieldSet(this, _BlockStack_accessibilityLabel_accessor_storage, value, "f"); }
        constructor() {
            super(shadowRoot);
            _BlockStack_accessibilityRole_accessor_storage.set(this, (__runInitializers(this, _instanceExtraInitializers), __runInitializers(this, _accessibilityRole_initializers, void 0)));
            _BlockStack_accessibilityLabel_accessor_storage.set(this, __runInitializers(this, _accessibilityLabel_initializers, void 0));
        }
    };
    _BlockStack_accessibilityRole_accessor_storage = new WeakMap();
    _BlockStack_accessibilityLabel_accessor_storage = new WeakMap();
    __setFunctionName(_classThis, "BlockStack");
    (() => {
        var _a;
        const _metadata = typeof Symbol === "function" && Symbol.metadata ? Object.create((_a = _classSuper[Symbol.metadata]) !== null && _a !== void 0 ? _a : null) : void 0;
        _accessibilityRole_decorators = [reactive(stringEnum(Object.keys(ACCESSIBILITY_ELEMENT_MAP)))];
        _accessibilityLabel_decorators = [reactive(string())];
        __esDecorate(_classThis, null, _accessibilityRole_decorators, { kind: "accessor", name: "accessibilityRole", static: false, private: false, access: { has: obj => "accessibilityRole" in obj, get: obj => obj.accessibilityRole, set: (obj, value) => { obj.accessibilityRole = value; } }, metadata: _metadata }, _accessibilityRole_initializers, _instanceExtraInitializers);
        __esDecorate(_classThis, null, _accessibilityLabel_decorators, { kind: "accessor", name: "accessibilityLabel", static: false, private: false, access: { has: obj => "accessibilityLabel" in obj, get: obj => obj.accessibilityLabel, set: (obj, value) => { obj.accessibilityLabel = value; } }, metadata: _metadata }, _accessibilityLabel_initializers, _instanceExtraInitializers);
        __esDecorate(null, _classDescriptor = { value: _classThis }, _classDecorators, { kind: "class", name: _classThis.name, metadata: _metadata }, null, _classExtraInitializers);
        _classThis = _classDescriptor.value;
        if (_metadata) Object.defineProperty(_classThis, Symbol.metadata, { enumerable: true, configurable: true, writable: true, value: _metadata });
        __runInitializers(_classThis, _classExtraInitializers);
    })();
    return _classThis;
})();

export { BlockStack as default };
