import { createStylesheet, y, __setFunctionName, __esDecorate, __runInitializers, PreactCustomElement, __classPrivateFieldGet, __classPrivateFieldSet } from '../../chunks/index.js';
import { reactive, define } from '../../chunks/define.js';
import { string } from '../../chunks/string.js';
import { tagName } from './meta.js';
import { classMap } from '../../chunks/classMap.js';
import { stringEnum } from '../../chunks/stringEnum.js';

const shadow = createStylesheet("@keyframes rotation{to{transform:rotate(1turn)}}svg{animation:rotation 500ms infinite linear}.size-small{height:1.25rem;width:1.25rem}.size-large{height:2.75rem;width:2.75rem}.tone-inherit{fill:inherit}.tone-default{fill:#303030}");

const largeSpinnerIconPath = `
    <path d="M15.542 1.487A21.507 21.507 0 00.5 22c0 11.874 9.626 21.5 21.5 21.5 9.847 0 18.364-6.675 20.809-16.072a1.5 1.5 0 00-2.904-.756C37.803 34.755 30.473 40.5 22 40.5 11.783 40.5 3.5 32.217 3.5 22c0-8.137 5.3-15.247 12.942-17.65a1.5 1.5 0 10-.9-2.863z"></path>
`;
const smallSpinnerIconPath = `
    <path d="M7.229 1.173a9.25 9.25 0 1011.655 11.412 1.25 1.25 0 10-2.4-.698 6.75 6.75 0 11-8.506-8.329 1.25 1.25 0 10-.75-2.385z"></path>
`;
function ShadowRoot(element) {
    const { tone, size, accessibilityLabel } = element;
    const isSmall = size === 'small-100';
    const svgClasses = classMap({
        'size-large': !isSmall,
        'size-small': isSmall,
        'tone-inherit': tone === 'inherit',
        'tone-default': tone === 'default',
    });
    const svgPath = isSmall ? smallSpinnerIconPath : largeSpinnerIconPath;
    const viewBow = isSmall ? '0 0 20 20' : '0 0 44 44';
    return (y("span", { "aria-label": accessibilityLabel },
        y("svg", { class: svgClasses, viewBox: viewBow, dangerouslySetInnerHTML: { __html: svgPath } })));
}

const shadowRoot = /*#__PURE__*/Object.freeze(/*#__PURE__*/Object.defineProperty({
      __proto__: null,
      ShadowRoot,
      styles: shadow
}, Symbol.toStringTag, { value: 'Module' }));

const sizeScale = [
    'small-300',
    'small-200',
    'small-100',
    'base',
    'large-100',
    'large-200',
    'large-300',
];

let ProgressIndicator = (() => {
    var _ProgressIndicator_accessibilityLabel_accessor_storage, _ProgressIndicator_tone_accessor_storage, _ProgressIndicator_size_accessor_storage, _ProgressIndicator_variant_accessor_storage;
    let _classDecorators = [define(tagName)];
    let _classDescriptor;
    let _classExtraInitializers = [];
    let _classThis;
    let _classSuper = PreactCustomElement;
    let _instanceExtraInitializers = [];
    let _accessibilityLabel_decorators;
    let _accessibilityLabel_initializers = [];
    let _tone_decorators;
    let _tone_initializers = [];
    let _size_decorators;
    let _size_initializers = [];
    let _variant_decorators;
    let _variant_initializers = [];
    _classThis = class extends _classSuper {
        get accessibilityLabel() { return __classPrivateFieldGet(this, _ProgressIndicator_accessibilityLabel_accessor_storage, "f"); }
        set accessibilityLabel(value) { __classPrivateFieldSet(this, _ProgressIndicator_accessibilityLabel_accessor_storage, value, "f"); }
        get tone() { return __classPrivateFieldGet(this, _ProgressIndicator_tone_accessor_storage, "f"); }
        set tone(value) { __classPrivateFieldSet(this, _ProgressIndicator_tone_accessor_storage, value, "f"); }
        get size() { return __classPrivateFieldGet(this, _ProgressIndicator_size_accessor_storage, "f"); }
        set size(value) { __classPrivateFieldSet(this, _ProgressIndicator_size_accessor_storage, value, "f"); }
        get variant() { return __classPrivateFieldGet(this, _ProgressIndicator_variant_accessor_storage, "f"); }
        set variant(value) { __classPrivateFieldSet(this, _ProgressIndicator_variant_accessor_storage, value, "f"); }
        constructor() {
            super(shadowRoot);
            _ProgressIndicator_accessibilityLabel_accessor_storage.set(this, (__runInitializers(this, _instanceExtraInitializers), __runInitializers(this, _accessibilityLabel_initializers, void 0)));
            _ProgressIndicator_tone_accessor_storage.set(this, __runInitializers(this, _tone_initializers, void 0));
            _ProgressIndicator_size_accessor_storage.set(this, __runInitializers(this, _size_initializers, void 0));
            _ProgressIndicator_variant_accessor_storage.set(this, __runInitializers(this, _variant_initializers, void 0));
        }
    };
    _ProgressIndicator_accessibilityLabel_accessor_storage = new WeakMap();
    _ProgressIndicator_tone_accessor_storage = new WeakMap();
    _ProgressIndicator_size_accessor_storage = new WeakMap();
    _ProgressIndicator_variant_accessor_storage = new WeakMap();
    __setFunctionName(_classThis, "ProgressIndicator");
    (() => {
        var _a;
        const _metadata = typeof Symbol === "function" && Symbol.metadata ? Object.create((_a = _classSuper[Symbol.metadata]) !== null && _a !== void 0 ? _a : null) : void 0;
        _accessibilityLabel_decorators = [reactive(string(), { attribute: 'accessibility-label' })];
        _tone_decorators = [reactive(stringEnum(['default', 'inherit']))];
        _size_decorators = [reactive(stringEnum(sizeScale, { defaultValue: 'base' }))];
        _variant_decorators = [reactive(string())];
        __esDecorate(_classThis, null, _accessibilityLabel_decorators, { kind: "accessor", name: "accessibilityLabel", static: false, private: false, access: { has: obj => "accessibilityLabel" in obj, get: obj => obj.accessibilityLabel, set: (obj, value) => { obj.accessibilityLabel = value; } }, metadata: _metadata }, _accessibilityLabel_initializers, _instanceExtraInitializers);
        __esDecorate(_classThis, null, _tone_decorators, { kind: "accessor", name: "tone", static: false, private: false, access: { has: obj => "tone" in obj, get: obj => obj.tone, set: (obj, value) => { obj.tone = value; } }, metadata: _metadata }, _tone_initializers, _instanceExtraInitializers);
        __esDecorate(_classThis, null, _size_decorators, { kind: "accessor", name: "size", static: false, private: false, access: { has: obj => "size" in obj, get: obj => obj.size, set: (obj, value) => { obj.size = value; } }, metadata: _metadata }, _size_initializers, _instanceExtraInitializers);
        __esDecorate(_classThis, null, _variant_decorators, { kind: "accessor", name: "variant", static: false, private: false, access: { has: obj => "variant" in obj, get: obj => obj.variant, set: (obj, value) => { obj.variant = value; } }, metadata: _metadata }, _variant_initializers, _instanceExtraInitializers);
        __esDecorate(null, _classDescriptor = { value: _classThis }, _classDecorators, { kind: "class", name: _classThis.name, metadata: _metadata }, null, _classExtraInitializers);
        _classThis = _classDescriptor.value;
        if (_metadata) Object.defineProperty(_classThis, Symbol.metadata, { enumerable: true, configurable: true, writable: true, value: _metadata });
        __runInitializers(_classThis, _classExtraInitializers);
    })();
    return _classThis;
})();

export { ProgressIndicator as default };
