import { useLayoutEffect } from 'preact/hooks';
type Params = Parameters<typeof useLayoutEffect>;
/**
 * Accepts a function that contains imperative, possibly effectful code.
 * Use this to read layout from the DOM and re-render before the next paint.
 *
 * This should be used instead of `useLayoutEffect`,
 * as it waits for changes from other frameworks, and other framework instances (e.g. other Preact-driven components).
 *
 * Prefer the standard `useEffect` hook when possible to avoid blocking visual updates.
 *
 * @param effect Imperative function that can return a cleanup function
 * @param inputs If present, effect will only activate if the values in the list change (using ===).
 */
export default function useBeforeNextPaint(effect: Params[0], inputs: Params[1]): void;
export {};
