/**
 * Restrict values to a set of strings, or (optionally) null.
 *
 * @param values Valid values for the enum. The first item is the default.
 * @returns
 */
function stringEnum(values, { defaultValue } = {}) {
    const valuesSet = new Set(values);
    const isValidValue = (value) => 
    // @ts-expect-error https://github.com/microsoft/TypeScript/issues/42641
    valuesSet.has(value);
    return {
        defaultValue: defaultValue !== null && defaultValue !== void 0 ? defaultValue : values[0],
        parseAttribute(value) {
            if (!isValidValue(value))
                return null;
            return value;
        },
        set(value, setter) {
            if (!isValidValue(value))
                return;
            setter(value);
        },
    };
}

export { stringEnum };
