import { PreactCustomElement } from '../../PreactCustomElement';
import { tagName } from './meta';
import { type AccessibilityRole } from '../../utils/accessibilityHelpers';
export interface SharedProps {
    /**
     * Sets the semantic meaning of the component’s content. When set,
     * the role will be used by assistive technologies to help users
     * navigate the page.
     *
     * @defaultValue 'generic'
     */
    accessibilityRole: AccessibilityRole;
    /**
     * A label that describes the purpose or contents of the element. When set,
     * it will be announced to buyers using assistive technologies and will
     * provide them with more context.
     */
    accessibilityLabel: string;
}
export default class BlockStack extends PreactCustomElement implements SharedProps {
    accessor accessibilityRole: AccessibilityRole;
    accessor accessibilityLabel: string;
    constructor();
}
declare global {
    interface HTMLElementTagNameMap {
        [tagName]: BlockStack;
    }
}
declare module 'preact' {
    namespace createElement.JSX {
        interface IntrinsicElements {
            [tagName]: Partial<BlockStack>;
        }
    }
}
