import { PreactCustomElement } from '../../PreactCustomElement';
import { tagName } from './meta';
import { type SizeScaleType } from '../../tokens/sizeScale';
export interface SharedProps {
    /**
     * A label used for users using assistive technologies. When set, any
     * `children` supplied to this component will not be announced to screen reader users.
     */
    accessibilityLabel: string;
    /**
     * The size of the component.
     */
    size: SizeScaleType;
    /**
     * Set the color of the progress indicator.
     *
     * - `inherit` will take the color value from its parent,
     * giving the progress indicator a monochrome appearance.
     *
     * @defaultValue 'default'
     */
    tone: 'inherit' | 'default';
    /**
     * Style of the progress indicator
     * @default 'spinner'
     *
     * TODO: Add 'bar'
     */
    variant: 'spinner';
}
export default class ProgressIndicator extends PreactCustomElement implements SharedProps {
    accessor accessibilityLabel: string;
    accessor tone: SharedProps['tone'];
    accessor size: SharedProps['size'];
    accessor variant: SharedProps['variant'];
    constructor();
}
declare global {
    interface HTMLElementTagNameMap {
        [tagName]: ProgressIndicator;
    }
}
declare module 'preact' {
    namespace createElement.JSX {
        interface IntrinsicElements {
            [tagName]: Partial<ProgressIndicator>;
        }
    }
}
