import { PreactCustomElement, createStylesheet, y as y$1, g, __setFunctionName, __esDecorate, __runInitializers, __classPrivateFieldGet, __classPrivateFieldSet } from '../../chunks/index.js';
import { define, reactive } from '../../chunks/reactive.js';
import { boolean } from '../../chunks/boolean.js';
import { string } from '../../chunks/string.js';
import { tagName } from './meta.js';
import { y, _, h, F, T, p } from '../../chunks/hooks.js';
import { classMap } from '../../chunks/classMap.js';

const isInRange = (hour, minute, rangeSpec) => {
    if (!rangeSpec)
        return true;
    const ranges = rangeSpec.split(',');
    const totalMinutes = hour * 60 + Number(minute);
    return ranges.some((range) => {
        const isRange = range.match(/\//);
        if (isRange) {
            const match = range.match(/(\d{2}):(\d{2})\s?\/\s?(\d{2}):(\d{2})/) || [];
            if (match) {
                const [, hour1, minute1, hour2, minute2] = match;
                const totalMinutes1 = Number(hour1) * 60 + Number(minute1);
                const totalMinutes2 = Number(hour2) * 60 + Number(minute2);
                return totalMinutes >= totalMinutes1 && totalMinutes < totalMinutes2;
            }
        }
        else {
            const match = range.match(/(\d{2}):(\d{2})/);
            if (match) {
                const [, hour, minutes] = match;
                return totalMinutes === Number(hour) * 60 + Number(minutes);
            }
        }
    });
};
const isSelected = (hour, minute, selectedTime) => {
    if (!selectedTime)
        return false;
    const [, selectedHour, selectedMinute] = selectedTime.match(/(\d{1,2}):(\d{1,2})/) || [];
    return (hour === Number(selectedHour) &&
        (minute !== undefined ? Number(selectedMinute) === minute : true));
};
const formatHour = (hour, minute) => {
    const minuteString = minute !== undefined ? `:${pad(minute)}` : '';
    const period = hour < 12 || hour === 24 ? 'AM' : 'PM';
    const adjustedHour = hour === 0 || hour === 12 ? 12 : hour % 12;
    return `${adjustedHour}${minuteString} ${period}`;
};
const pad = (val) => { var _a; return (_a = val === null || val === void 0 ? void 0 : val.toString()) === null || _a === void 0 ? void 0 : _a.padStart(2, '0'); };

let pendingAnimationFrame = false;
const pendingCallbacks = new Set();
/**
 * Accepts a function that contains imperative, possibly effectful code.
 * Use this to read layout from the DOM and re-render before the next paint.
 *
 * This should be used instead of `useLayoutEffect`,
 * as it waits for changes from other frameworks, and other framework instances (e.g. other Preact-driven components).
 *
 * Prefer the standard `useEffect` hook when possible to avoid blocking visual updates.
 *
 * @param effect Imperative function that can return a cleanup function
 * @param inputs If present, effect will only activate if the values in the list change (using ===).
 */
function useBeforeNextPaint(effect, inputs) {
    // useLayoutEffect is needed to define this hook.
    // eslint-disable-next-line local-rules/no-preact-useLayoutEffect
    y(() => {
        let cleanupCallback;
        const pendingCallback = () => {
            cleanupCallback = effect();
        };
        pendingCallbacks.add(pendingCallback);
        if (!pendingAnimationFrame) {
            pendingAnimationFrame = true;
            // Run all of the effects via requestAnimationFrame, which will be before the next paint.
            requestAnimationFrame(() => {
                pendingAnimationFrame = false;
                // All of the callbacks run in a single rAF callback to avoid microtasks running between effects.
                // Rendering happens in microtasks, so this means all effects run before the next render,
                // which avoids layout thrashing caused by interleaving style reads and writes.
                for (const callback of pendingCallbacks) {
                    try {
                        callback();
                    }
                    catch (error) {
                        console.error(error);
                    }
                }
                pendingCallbacks.clear();
            });
        }
        return () => {
            // It's possible that this runs before pendingCallback is called.
            // In which case we don't want to call it.
            pendingCallbacks.delete(pendingCallback);
            cleanupCallback === null || cleanupCallback === void 0 ? void 0 : cleanupCallback();
        };
        // Because these are passthrough inputs, 'effect' will be the correct reference (updated when `inputs` updates).
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, inputs);
}

/**
 * Adopt a stylesheet into the parent shadow root.
 *
 * @param ref A ref to an element that's always present in the component.
 * @param styles The stylesheet to add to the shadow root.
 */
function useAdoptStyles(ref, styles) {
    useBeforeNextPaint(() => {
        if (!ref.current)
            return;
        const root = ref.current.getRootNode();
        if (!(root instanceof ShadowRoot))
            return;
        // Nice easy modern browser way:
        if (styles instanceof CSSStyleSheet) {
            if (root.adoptedStyleSheets.includes(styles))
                return;
            root.adoptedStyleSheets = [...root.adoptedStyleSheets, styles];
            return;
        }
        // Old browser way:
        if (root.host instanceof PreactCustomElement) {
            root.host._addLegacyStyleComponent(styles);
        }
    }, []);
}

const styles = createStylesheet("header.disabled{color:#b5b5b5}header .back-button{background:0 0;border:0;cursor:pointer;padding:.25rem;border-radius:.75rem}header .back-button:hover{background-color:#f7f7f7}header .back-button svg{opacity:.5;height:18px}header .back-button:hover svg{opacity:1}.hours-list{margin:0;list-style:none;padding:0 .25rem .25rem;font-weight:450;overflow-y:scroll;max-height:80vh;max-height:calc(100vh - 62px)}.hours-list .hour-item{padding:.5rem;border-radius:.5rem;-webkit-user-select:none;user-select:none;border:0;display:flex;width:100%;background-color:unset;font:inherit;text-align:inherit;justify-content:space-between;align-items:center}.hours-list .hour-item:hover{cursor:pointer;background-color:#f7f7f7}.hours-list .hour-item svg{margin-right:.25rem}.hours-list .hour-item:focus-within:not(.disabled,.selected){background-color:#f3f3f3}.hours-list .hour-item.disabled{pointer-events:none;color:#b5b5b5}.hours-list .hour-item.disabled.selected,.hours-list .hour-item.selected{background-color:#ebebeb}.shadow{position:absolute;left:0;z-index:100;width:100%;height:6px;transition:opacity .2s}.shadow--top{top:0;background:linear-gradient(180deg,rgb(26 26 26/5%),transparent)}.shadow--bottom{bottom:0;background:linear-gradient(0deg,rgb(26 26 26/7.5%),transparent)}");

const START_TIME = 0;
const TimePickerMain = ({ element }) => {
    const { steps: stepsAttribute, allowRanges, disallowRanges, disabled, defaultValue, value, } = element;
    const listRef = _();
    useAdoptStyles(listRef, styles);
    const steps = stepsAttribute || '00:15';
    const [selectedHour, setSelectedHour] = h();
    const [shadowTop, setShadowTop] = h(0);
    const [shadowBottom, setShadowBottom] = h(1);
    const stepsInMinutes = F(() => {
        const match = steps.match(/(\d{2}):(\d{2})/);
        if (match) {
            const [, hours, minutes] = match;
            return Number(hours) * 60 + Number(minutes);
        }
        return 0;
    }, [steps]);
    const hours = F(() => {
        const times = [];
        const hours = Array.from({ length: 24 }, (_, i) => (i + START_TIME) % 24);
        const minutes = Array.from({ length: 60 / stepsInMinutes }, (_, i) => i * stepsInMinutes);
        for (const hour of hours) {
            for (const minute of minutes) {
                times.push({
                    hour,
                    minute,
                    disabled: Boolean(disabled ||
                        (allowRanges && !isInRange(hour, minute, allowRanges)) ||
                        (disallowRanges && isInRange(hour, minute, disallowRanges))),
                });
            }
        }
        return times;
    }, [stepsInMinutes, allowRanges, disallowRanges, disabled]);
    const displayHours = F(() => {
        if (typeof selectedHour === 'number') {
            return hours.filter(({ hour }) => hour === selectedHour);
        }
        else if (hours.length > 60) {
            const accHours = [];
            for (const time of hours) {
                const prevIndex = accHours.length - 1;
                const prev = accHours[prevIndex];
                if (!prev || prev.hour !== time.hour) {
                    accHours.push({ hour: time.hour, disabled: time.disabled });
                }
                else if (prev.hour === time.hour &&
                    prev.disabled === true &&
                    disabled === false) {
                    accHours[prevIndex].disabled = time.disabled;
                }
            }
            return accHours;
        }
        else {
            return hours;
        }
    }, [hours, selectedHour, disabled]);
    const resetView = T(() => {
        var _a, _b, _c;
        const hourItems = Array.from(((_a = listRef === null || listRef === void 0 ? void 0 : listRef.current) === null || _a === void 0 ? void 0 : _a.querySelectorAll('.hour-item')) || []);
        const focusElement = hourItems.find((el) => el.classList.contains('selected') &&
            !el.classList.contains('disabled')) || hourItems.find((el) => !el.classList.contains('disabled'));
        if (listRef.current) {
            (_b = listRef.current) === null || _b === void 0 ? void 0 : _b.scrollTo(0, (focusElement === null || focusElement === void 0 ? void 0 : focusElement.offsetTop) || 0);
        }
        (_c = focusElement === null || focusElement === void 0 ? void 0 : focusElement.focus) === null || _c === void 0 ? void 0 : _c.call(focusElement);
    }, []);
    const onHourClick = T((hour, minute) => {
        if (hours.length > 60 &&
            selectedHour === undefined &&
            minute === undefined) {
            setSelectedHour(hour);
            // Had to use setTimeout to make sure the listRef has updated before calculating the position
            setTimeout(resetView, 0);
        }
        else {
            const newValue = `${pad(hour)}:${pad(minute)}`;
            if (element.value !== newValue) {
                element.value = newValue;
                element.dispatchEvent(new Event('input'));
            }
        }
    }, [hours.length, selectedHour, resetView, element]);
    const onBackClick = T(() => {
        setSelectedHour(undefined);
        setTimeout(resetView, 0);
    }, [resetView]);
    const handleScroll = T((e) => {
        const { scrollTop, scrollHeight, offsetHeight } = e.target;
        setShadowTop(Number(scrollTop > 0));
        setShadowBottom(Number(scrollHeight - offsetHeight > scrollTop));
    }, []);
    p(() => {
        resetView();
    }, [resetView]);
    const headerMarkup = hours.length > 60 ? (selectedHour !== undefined ? (y$1("header", null,
        y$1("button", { class: 'back-button', "aria-label": "back", tabIndex: 0, onClick: onBackClick },
            y$1("svg", { viewBox: "0 0 20 16", xmlns: "http://www.w3.org/2000/svg" },
                y$1("path", { "fill-rule": "evenodd", d: "M16.75 10a.75.75 0 0 1-.75.75h-9.69l2.72 2.72a.75.75 0 0 1-1.06 1.06l-4-4a.75.75 0 0 1 0-1.06l4-4a.75.75 0 0 1 1.06 1.06l-2.72 2.72h9.69a.75.75 0 0 1 .75.75Z" }))),
        y$1("h4", null, "Select minutes"))) : (y$1("header", null,
        y$1("h4", null, "Select hours")))) : null;
    return (y$1(g, null,
        headerMarkup,
        y$1("div", { style: { position: 'relative' } },
            y$1("div", { class: "shadow shadow--top", style: { opacity: shadowTop } }),
            y$1("div", { class: "shadow shadow--bottom", style: { opacity: shadowBottom } }),
            y$1("ul", { ref: listRef, class: 'hours-list', onScroll: handleScroll, "aria-live": "assertive" }, displayHours.map(({ hour, minute, disabled }) => {
                const selected = isSelected(hour, minute, value || defaultValue);
                const formatedHour = formatHour(hour, minute);
                return (y$1("li", { key: `${hour}:${minute}` },
                    y$1("button", { "aria-label": `${formatedHour} ${selected ? 'selected' : disabled ? 'disabled' : ''}`, tabIndex: disabled ? -1 : 0, class: `hour-item ${classMap({
                            disabled,
                            selected,
                        })}`, onClick: () => {
                            if (!disabled)
                                onHourClick(hour, minute);
                        } },
                        formatedHour,
                        selected && (y$1("svg", { width: "12", height: "9", viewBox: "0 0 12 9", fill: "none", xmlns: "http://www.w3.org/2000/svg" },
                            y$1("path", { "fill-rule": "evenodd", "clip-rule": "evenodd", d: "M11.7803 0.96967C12.0732 1.26256 12.0732 1.73744 11.7803 2.03033L5.28033 8.53033C4.98744 8.82322 4.51256 8.82322 4.21967 8.53033L0.96967 5.28033C0.676777 4.98744 0.676777 4.51256 0.96967 4.21967C1.26256 3.92678 1.73744 3.92678 2.03033 4.21967L4.75 6.93934L10.7197 0.96967C11.0126 0.676777 11.4874 0.676777 11.7803 0.96967Z", fill: "#1A1A1A" }))))));
            })))));
};

const oldDynamicImport0 = /*#__PURE__*/Object.freeze(/*#__PURE__*/Object.defineProperty({
    __proto__: null,
    default: TimePickerMain
}, Symbol.toStringTag, { value: 'Module' }));

/**
 * Create a lazy-loaded component.
 *
 * @param factory A function that returns a promise for an object with a `default` property that's the target component.
 * @returns A component with the same props as the target component, but with an additional `fallback` prop.
 *
 * @example
 * ```tsx
 * const LazyComponent = lazyComponent(() => import('./MyComponent'));
 *
 * function App() {
 *  return <LazyComponent fallback={<div>Loading...</div>} other="props" go="here" />;
 * }
 * ```
 */
function lazyComponent(factory) {
    let componentPromise;
    let componentStateSetter;
    const LazyComponent = (props) => {
        const { fallback, ...rest } = props;
        const [Component, setComponent] = h(componentStateSetter);
        if (!componentStateSetter) {
            if (!componentPromise) {
                componentPromise = factory().then((m) => m.default);
            }
            // Chill, errors are handled in the function.
            // eslint-disable-next-line @typescript-eslint/no-floating-promises
            (async () => {
                const Component = await componentPromise.catch((error) => {
                    // As a fallback, return a callback that throws the error
                    return () => {
                        throw error;
                    };
                });
                // setComponent & useState have an overload that accepts a function.
                // Since the value we're setting is also a function (components are functions),
                // we need to wrap it in a function.
                componentStateSetter = () => Component;
                setComponent(componentStateSetter);
            })();
        }
        if (Component)
            return y$1(Component, { ...rest });
        return fallback !== null && fallback !== void 0 ? fallback : null;
    };
    return LazyComponent;
}

const shadow$1 = createStylesheet(":host{font-family:\"Inter\",-apple-system,BlinkMacSystemFont,\"San Francisco\",\"Segoe UI\",Roboto,\"Helvetica Neue\",sans-serif}.time-picker{border-radius:.75rem;box-shadow:0 .25rem .375rem -.125rem rgba(26,26,26,.2);border:.0625rem solid #e3e3e3;overflow:hidden;font-size:.8125rem}header{display:flex;align-items:center;gap:5px;padding:.25rem .5rem}header h4{padding:.5rem 0;margin:0;position:sticky;top:0;background-color:#fff;font-weight:650}.time-picker.disabled header{color:#b5b5b5}");

const LazyTimePickerMain = lazyComponent(() => Promise.resolve(oldDynamicImport0));
function ShadowRoot$1(element) {
    const { disabled } = element;
    return (y$1("div", { class: classMap({
            'time-picker': true,
            disabled,
        }) },
        y$1(LazyTimePickerMain, { fallback: y$1("header", null,
                y$1("h4", null, "Select\u2026")), element: element })));
}

const shadow = /*#__PURE__*/Object.freeze(/*#__PURE__*/Object.defineProperty({
    __proto__: null,
    ShadowRoot: ShadowRoot$1,
    styles: shadow$1
}, Symbol.toStringTag, { value: 'Module' }));

let TimePicker = (() => {
    var _TimePicker_allowRanges_accessor_storage, _TimePicker_disallowRanges_accessor_storage, _TimePicker_steps_accessor_storage, _TimePicker_disabled_accessor_storage, _TimePicker_defaultValue_accessor_storage, _TimePicker_value_accessor_storage;
    let _classDecorators = [define(tagName)];
    let _classDescriptor;
    let _classExtraInitializers = [];
    let _classThis;
    let _classSuper = PreactCustomElement;
    let _instanceExtraInitializers = [];
    let _allowRanges_decorators;
    let _allowRanges_initializers = [];
    let _disallowRanges_decorators;
    let _disallowRanges_initializers = [];
    let _steps_decorators;
    let _steps_initializers = [];
    let _disabled_decorators;
    let _disabled_initializers = [];
    let _defaultValue_decorators;
    let _defaultValue_initializers = [];
    let _value_decorators;
    let _value_initializers = [];
    _classThis = class extends _classSuper {
        get allowRanges() { return __classPrivateFieldGet(this, _TimePicker_allowRanges_accessor_storage, "f"); }
        set allowRanges(value) { __classPrivateFieldSet(this, _TimePicker_allowRanges_accessor_storage, value, "f"); }
        get disallowRanges() { return __classPrivateFieldGet(this, _TimePicker_disallowRanges_accessor_storage, "f"); }
        set disallowRanges(value) { __classPrivateFieldSet(this, _TimePicker_disallowRanges_accessor_storage, value, "f"); }
        get steps() { return __classPrivateFieldGet(this, _TimePicker_steps_accessor_storage, "f"); }
        set steps(value) { __classPrivateFieldSet(this, _TimePicker_steps_accessor_storage, value, "f"); }
        get disabled() { return __classPrivateFieldGet(this, _TimePicker_disabled_accessor_storage, "f"); }
        set disabled(value) { __classPrivateFieldSet(this, _TimePicker_disabled_accessor_storage, value, "f"); }
        get defaultValue() { return __classPrivateFieldGet(this, _TimePicker_defaultValue_accessor_storage, "f"); }
        set defaultValue(value) { __classPrivateFieldSet(this, _TimePicker_defaultValue_accessor_storage, value, "f"); }
        get value() { return __classPrivateFieldGet(this, _TimePicker_value_accessor_storage, "f"); }
        set value(value) { __classPrivateFieldSet(this, _TimePicker_value_accessor_storage, value, "f"); }
        constructor() {
            super(shadow);
            _TimePicker_allowRanges_accessor_storage.set(this, (__runInitializers(this, _instanceExtraInitializers), __runInitializers(this, _allowRanges_initializers, void 0)));
            _TimePicker_disallowRanges_accessor_storage.set(this, __runInitializers(this, _disallowRanges_initializers, void 0));
            _TimePicker_steps_accessor_storage.set(this, __runInitializers(this, _steps_initializers, void 0));
            _TimePicker_disabled_accessor_storage.set(this, __runInitializers(this, _disabled_initializers, void 0));
            _TimePicker_defaultValue_accessor_storage.set(this, __runInitializers(this, _defaultValue_initializers, void 0));
            _TimePicker_value_accessor_storage.set(this, __runInitializers(this, _value_initializers, void 0));
        }
    };
    _TimePicker_allowRanges_accessor_storage = new WeakMap();
    _TimePicker_disallowRanges_accessor_storage = new WeakMap();
    _TimePicker_steps_accessor_storage = new WeakMap();
    _TimePicker_disabled_accessor_storage = new WeakMap();
    _TimePicker_defaultValue_accessor_storage = new WeakMap();
    _TimePicker_value_accessor_storage = new WeakMap();
    __setFunctionName(_classThis, "TimePicker");
    (() => {
        var _a;
        const _metadata = typeof Symbol === "function" && Symbol.metadata ? Object.create((_a = _classSuper[Symbol.metadata]) !== null && _a !== void 0 ? _a : null) : void 0;
        _allowRanges_decorators = [reactive(string())];
        _disallowRanges_decorators = [reactive(string())];
        _steps_decorators = [reactive(string())];
        _disabled_decorators = [reactive(boolean())];
        _defaultValue_decorators = [reactive(string())];
        _value_decorators = [reactive(string())];
        __esDecorate(_classThis, null, _allowRanges_decorators, { kind: "accessor", name: "allowRanges", static: false, private: false, access: { has: obj => "allowRanges" in obj, get: obj => obj.allowRanges, set: (obj, value) => { obj.allowRanges = value; } }, metadata: _metadata }, _allowRanges_initializers, _instanceExtraInitializers);
        __esDecorate(_classThis, null, _disallowRanges_decorators, { kind: "accessor", name: "disallowRanges", static: false, private: false, access: { has: obj => "disallowRanges" in obj, get: obj => obj.disallowRanges, set: (obj, value) => { obj.disallowRanges = value; } }, metadata: _metadata }, _disallowRanges_initializers, _instanceExtraInitializers);
        __esDecorate(_classThis, null, _steps_decorators, { kind: "accessor", name: "steps", static: false, private: false, access: { has: obj => "steps" in obj, get: obj => obj.steps, set: (obj, value) => { obj.steps = value; } }, metadata: _metadata }, _steps_initializers, _instanceExtraInitializers);
        __esDecorate(_classThis, null, _disabled_decorators, { kind: "accessor", name: "disabled", static: false, private: false, access: { has: obj => "disabled" in obj, get: obj => obj.disabled, set: (obj, value) => { obj.disabled = value; } }, metadata: _metadata }, _disabled_initializers, _instanceExtraInitializers);
        __esDecorate(_classThis, null, _defaultValue_decorators, { kind: "accessor", name: "defaultValue", static: false, private: false, access: { has: obj => "defaultValue" in obj, get: obj => obj.defaultValue, set: (obj, value) => { obj.defaultValue = value; } }, metadata: _metadata }, _defaultValue_initializers, _instanceExtraInitializers);
        __esDecorate(_classThis, null, _value_decorators, { kind: "accessor", name: "value", static: false, private: false, access: { has: obj => "value" in obj, get: obj => obj.value, set: (obj, value) => { obj.value = value; } }, metadata: _metadata }, _value_initializers, _instanceExtraInitializers);
        __esDecorate(null, _classDescriptor = { value: _classThis }, _classDecorators, { kind: "class", name: _classThis.name, metadata: _metadata }, null, _classExtraInitializers);
        _classThis = _classDescriptor.value;
        if (_metadata) Object.defineProperty(_classThis, Symbol.metadata, { enumerable: true, configurable: true, writable: true, value: _metadata });
        __runInitializers(_classThis, _classExtraInitializers);
    })();
    return _classThis;
})();

export { TimePicker as default };
