type oneTo9 = 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9;
type oneTo5 = 0 | 1 | 2 | 3 | 4 | 5;
type HH = `${0 | 1}${oneTo9}` | `2${0 | 1 | 2 | 3}`;
type SS = `${oneTo5}${oneTo9}`;
/**
 * A time in HH:MM format.
 */
export type TimeSpec = `${HH}:${SS}`;
export type TimeRangeSpec = string;
export interface SharedTimePickerProps {
    /**
     * Ranges of times selectable.
     *
     * If no range is specified, the allowed range is any time.
     */
    allowRanges: TimeRangeSpec;
    /**
     * Ranges of times not selectable. This subtracts from the allowed range specified.
     *
     * If no range is specified, nothing is subtracted from the allowed ranges.
     */
    disallowRanges: TimeRangeSpec;
    /**
     * The increments that can be selected.
     *
     * Increments are calculated from 00:00, regardless of allowed/disallowed ranges.
     *
     * @default '00:15'
     */
    steps: TimeSpec;
    /**
     * Whether the field is disabled.
     *
     * When disabled, the field cannot be interacted with, and isn't included in form data.
     *
     * https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/disabled
     */
    disabled: boolean;
    /**
     * The initial value, before interation.
     */
    defaultValue: TimeSpec;
    /**
     * The currently selected time.
     */
    value: TimeSpec;
}
export {};
