import { FunctionComponent, VNode } from 'preact';
interface LazyComponentProps {
    fallback?: VNode;
}
/**
 * Create a lazy-loaded component.
 *
 * @param factory A function that returns a promise for an object with a `default` property that's the target component.
 * @returns A component with the same props as the target component, but with an additional `fallback` prop.
 *
 * @example
 * ```tsx
 * const LazyComponent = lazyComponent(() => import('./MyComponent'));
 *
 * function App() {
 *  return <LazyComponent fallback={<div>Loading...</div>} other="props" go="here" />;
 * }
 * ```
 */
export default function lazyComponent<P extends object>(factory: () => Promise<{
    default: FunctionComponent<P>;
}>): FunctionComponent<LazyComponentProps & P>;
export {};
